#
# Implementing the analyses and simulations in the paper
# XXX (2017) Test for trend with a multinomial outcome.
#

# set to TRUE to rerun the simulations from scratch
rerunOverall <- FALSE
rerunIndiv <- FALSE

# load libraries
library(multiCA)
library(ggplot2)
theme_set(theme_bw(base_size = 12))
library(dplyr)
library(directlabels)
library(reshape2)

######################
## Stroke data example
######################
data(stroke)

# trend plot
stroke <-group_by(stroke, Year) %>% mutate(Prop = Freq / sum(Freq))
levels(stroke$Type) <- LETTERS[1:5]

(strk.pl <- ggplot(stroke, aes(x=Year, y=Prop, linetype=Type, label=Type, group=Type)) +
  geom_line() + geom_dl(method=list("last.qp", hjust=-0.5)) +
  scale_linetype_discrete(guide="none") +
  scale_x_continuous(limits=c(2003,2011), breaks=unique(stroke$Year)) +
  scale_y_continuous("Proportion of given etiology"))
 
pdf(file="StrokeTrend.pdf", width=8, height=5)
  strk.pl
dev.off()

# multinomial trend test
multiCA.test(Type ~ Year, data=stroke, weights=Freq)

# various multiple testing adjustment
p.unadj <- multiCA.test(Type ~ Year, data=stroke, weights=Freq, 
                        p.adjust.method="none")$individual
p.closed.set <- multiCA.test(Type ~ Year, data=stroke, weights=Freq, 
                             p.adjust.method="closed.set")$individual
p.holm.shaffer <- multiCA.test(Type ~ Year, data=stroke, weights=Freq, 
                             p.adjust.method="Holm-Shaffer")$individual
p.single <- multiCA.test(Type ~ Year, data=stroke, weights=Freq, 
                               p.adjust.method="single")$individual
p.Westfall <- multiCA.test(Type ~ Year, data=stroke, weights=Freq, 
                               p.adjust.method="Westfall")$individual
cbind(p.unadj, p.holm.shaffer, p.single, p.closed.set, p.Westfall)

###################
## Simulation study
###################

# load functions for generating random matrices with given NCP
source("SimMultiSlope.R")

# simulation for global tests
gen.data <- function(nvec, pmat){
  #generate counts, conditional on group sizes nvec
  res <- sapply(seq_along(nvec), function(i){
    rmultinom(1, size=nvec[i], prob=pmat[,i])})
  rownames(res) <- 1:nrow(res)
  res  
}

simulate.test <- function(nvec, pmat, sig.level=0.05){
  x <- gen.data(nvec=nvec, pmat=pmat)
  xres <- multiCA.test(x)$overall
  pval <- xres$p.value
  res <- (pval <= sig.level)
  invisible(list(result=res, p.values=pval))
}

# generate a new random matrix for every simulation
# all combinations of ncp and per-group sample size
sim.set <- function(ncp.vec, ss.vec, K, nc, R=1000, ngrid=21, maxit=100) {
  simone <-  function(ncp, n){
    nvec <- rep(n, nc)
    testp <- ran.pmatrix.ncp(ncp=ncp, K=K, nc=nc, nvec=nvec, ngrid=ngrid,
                             maxit=maxit)
    sig <- simulate.test(nvec=nvec, pmat=testp)$res
    calcpwr <- power.multiCA.test(N=sum(nvec), pmatrix=testp)$power
    c(ncp, n, calcpwr, sig)
  }
  simmany <- function(ncp, n){
    tmp <- replicate(R, tryCatch(simone(ncp, n), error=function(e)c(ncp,n,NA,NA)))
    rowMeans(tmp, na.rm=TRUE)
  }
  res <- mapply(simmany, rep(ncp.vec, each=length(ss.vec)), rep(ss.vec, length(ncp.vec)))
  resd <- as.data.frame(t(res))
  names(resd) <- c("ncp", "n", "power.calc", "power")
  resd$K <- K
  resd <- transform(resd, se = sqrt(power * (1-power)/R))
  resd
}

# ncp values to achieve power from 0.05 to 0.95
target.pwr <- seq(0.05, 0.95, by=0.05)
target.K <- 3
target.ncp3 <- sapply(target.pwr,
                      function(pw) cnonct(x = qchisq(0.05, df=target.K-1, lower=FALSE),
                                          p = 1-pw, 
                                          df = target.K-1))
target.K <- 5
target.ncp5 <- sapply(target.pwr,
                      function(pw) cnonct(x = qchisq(0.05, df=target.K-1, lower=FALSE),
                                          p = 1-pw, 
                                          df = target.K-1))
target.K <- 7
target.ncp7 <- sapply(target.pwr,
                      function(pw) cnonct(x = qchisq(0.05, df=target.K-1, lower=FALSE),
                                          p = 1-pw, 
                                          df = target.K-1))

if (rerunOverall){
  set.seed(346236)
  # under the null hypothesis
  s3a <- sim.set(ncp.vec=0, ss.vec=seq(10, 100, by=5), K=3, nc=5, ngrid=21, R=5000)
  s5a <- sim.set(ncp.vec=0, ss.vec=seq(10, 100, by=5), K=5, nc=5, ngrid=21, R=5000)
  s7a <- sim.set(ncp.vec=0, ss.vec=seq(10, 100, by=5), K=7, nc=5, ngrid=21, R=5000)

  # under the alternative hypothesis
  s3b <- sim.set(ncp.vec=target.ncp3, ss.vec=c(20, 50, 100), K=3, nc=5, ngrid=21, R=5000)
  s5b <- sim.set(ncp.vec=target.ncp5, ss.vec=c(20, 50, 100), K=5, nc=5, ngrid=21, R=5000)
  s7b <- sim.set(ncp.vec=target.ncp7, ss.vec=c(20, 50, 100), K=7, nc=5, ngrid=21, R=5000)
  
  simres.a <- rbind(s3a, s5a, s7a)
  simres.b <- rbind(s3b, s5b, s7b)
  save(simres.a, simres.b, file="SimOverall.RData")
} else {
  load(file="SimOverall.RData")
}


pdf(file="TypeISim.pdf", width=8, height=4)
ggplot(simres.a, aes(x=n, y=power))  + facet_grid(~K, labeller = label_both)  + 
  geom_pointrange(aes(ymin=power-1.96*se, ymax=power+1.96*se)) +
  geom_hline(yintercept=0.05, colour="red") +
  scale_y_continuous("Type I error", limits=c(0, 0.07),expand=c(0,0)) +
  scale_x_continuous("Sample size per group") 
dev.off()

pdf(file="PowerSim.pdf", width=10, height=8)
ggplot(simres.b, aes(x=power.calc, y=power-power.calc)) + 
  facet_grid(n~K, labeller = label_both) +
  geom_pointrange(aes(ymin=power-power.calc-1.96*se, ymax=power-power.calc+1.96*se)) +
  geom_abline(slope=0, intercept=0, colour="red") +
  scale_y_continuous("Bias") +
  scale_x_continuous("Target power") 
dev.off()



### 
# power of individual tests
simulate.test.indiv <- function(nvec, pmat, H0s, sig.level=0.05,
                                adj.method.list = c("Holm-Shaffer", "closed.set", "Westfall")){
  x <- gen.data(nvec=nvec, pmat=pmat)
  xres.list <- lapply(adj.method.list, 
                      function(am)multiCA.test(x, p.adjust.method = am)$individual)
  nsig <- sapply(xres.list, function(xr) sum(xr[-H0s] <= sig.level))  
  diffs <- outer(nsig, nsig, "-")
  diffs <- diffs[upper.tri(diffs)]
  H0sig <- sapply(xres.list, function(xr) any(xr[H0s] <= sig.level))
  c(H0sig, nsig, diffs)
}



# generate matrices with given ncp and L constant rows
# generate new random matrix for every data set

sim.set.indiv <- function(ncp.vec, K, L, nc, nvec, R=1000, ngrid=21, maxit=100, 
                          overall.ncp=TRUE,
                          adj.method.list = c("Holm-Shaffer", "closed.set", "Westfall"),
                          n.cores=1) {
  simone <- function(ncp){
    testp <- ran.pmatrix0.ncp(ncp=ncp, K=K, nc=nc, nvec=nvec, L=L, ngrid=ngrid,
                              maxit=maxit, overall.ncp=overall.ncp)
    p.alt.sum <- sum(testp[(L+1):K, 1])
    ncp0 <- ncp * p.alt.sum
    r <- simulate.test.indiv(nvec=nvec, pmat=testp, H0s=1:L,
                             adj.method.list = adj.method.list)
    c(ncp, ncp0, r)
  }
  n.methods <- length(adj.method.list)
  simone.r.length <- 2*n.methods + choose(n.methods, 2)
  
  simmany <- function(ncp){
    tmp <- replicate(R, tryCatch(simone(ncp), 
                                 error=function(e)c(ncp, rep(NA,simone.r.length+1))))
    avetmp <- rowMeans(tmp, na.rm=TRUE)
    diff.idx <- 2 + 2*n.methods + (1:choose(n.methods,2))
    avetmp <- c(avetmp, apply(tmp[diff.idx,], 1, sd)/sqrt(R))  # se.diff = sd(diff)/sqrt(n)
    avetmp
  }
  
  cl <- makeCluster(n.cores, type="FORK")
  res <- parSapply(cl, ncp.vec, function(ncp){simmany(ncp)})
  stopCluster(cl)
  
  resd <- as.data.frame( t(res))
  pairnames <- outer(1:n.methods, 1:n.methods, paste0)
  pairnames <- pairnames[upper.tri(pairnames)]
  names(resd) <- c("ncp","ncp0", 
                   paste0("H0sig", 1:n.methods),
                   paste0("nsig", 1:n.methods),
                   paste0("diff", pairnames),
                   paste0("se", pairnames))
  resd$K <- K
  resd$L <- L
  resd
}


adj.method.list0 <- c("Holm-Shaffer", "closed.set", "single", "Westfall")
adj.method.list0.labels <- c("Holm-Shaffer", "Closed set", "Single-step", "Westfall")
R0 <- 5000
if (rerunIndiv){
  set.seed(457437)
  r3 <- sim.set.indiv(ncp.vec=0:29, K=3, nc=5, nvec=rep(100, 5), ngrid=21,
                      R=R0, L=1, adj.method.list = adj.method.list0, n.cores=30)
  save(r3, file="SimIndiv4_r3.RData")
  r5 <- lapply(1:3, function(L){
    sim.set.indiv(ncp.vec=0:29, K=5, nc=5, nvec=rep(100, 5), ngrid=21,
                  R=R0, L=L, adj.method.list = adj.method.list0, n.cores=30)})
  save(r5, file="SimIndiv4_r5.RData")
  r7 <- lapply(1:5, function(L){
    sim.set.indiv(ncp.vec=0:29, K=7, nc=5, nvec=rep(100, 5), ngrid=21,
                  R=R0, L=L, adj.method.list = adj.method.list0, n.cores=30)})
  save(r7, file="SimIndiv4_r7.RData")
  
  rres <- do.call(rbind, c(list(r3), r5, r7))
  save(rres, file="SimIndiv4.RData")
} else {
  load(file="SimIndiv4.RData")
}

#plots
# type I error
rres2 <- melt(rres, id.var=c("K","L", "ncp","ncp0"), 
              measure.vars = c("H0sig1", "H0sig2", "H0sig3", "H0sig4"))
rres2 <- transform(rres2, 
                   Method = adj.method.list0.labels[unclass(variable)])

pdf(file="IndivTypeISim.pdf", width=8, height=8)
ggplot(rres2, aes(x=ncp, y=value,, linetype=factor(K-L), shape=factor(K-L))) + 
  geom_hline(yintercept=0.05, colour="red") +
  geom_ribbon(aes(ymin=value-sqrt(value*(1-value)/5000), 
                 ymax=value+sqrt(value*(1-value)/5000)), alpha=0.2, colour=NA,
             show.legend=FALSE)+
  geom_line() +
  geom_point() +
  facet_grid(Method~K, labeller = label_both) +
  scale_x_continuous("Non-centrality parameter") +
  scale_y_continuous("Familywise type I error", limit=c(0,0.06)) +
  scale_linetype_discrete("# of false hypotheses") +
  scale_shape_discrete("# of false hypotheses", solid=FALSE) +
  theme(legend.position="bottom")
dev.off()

# individual power
rres3a <- melt(rres, id.var=c("K","L", "ncp","ncp0"), 
               measure.vars = c("diff12", "diff13", "diff14"), value.name="diff") %>%
  mutate(Method = adj.method.list0.labels[unclass(variable)+1])
rres3b <- melt(rres, id.var=c("K","L", "ncp","ncp0"), 
               measure.vars = c("se12", "se13", "se14"), value.name="se") %>%
  mutate(Method = adj.method.list0.labels[unclass(variable)+1])
rres3 <- full_join(rres3a, rres3b, by=c("Method", "K","L", "ncp","ncp0"))
rres3 <- mutate(rres3, 
                PowerDiff = - diff / (K-L),
                PowerDiffSE = se / (K-L),
                NFalse = K-L,
                Group = paste("K=", K, "; # false =", NFalse))

pdf(file="IndivPowerSim.pdf", width=8, height=6)
ggplot(rres3, aes(x=ncp, y=PowerDiff, linetype=Method)) + 
  geom_hline(yintercept=0.0, colour="red") +
  geom_ribbon(aes(ymin=PowerDiff-1.96*PowerDiffSE, 
                  ymax=PowerDiff+1.96*PowerDiffSE), alpha=0.2, colour=NA,
              show.legend=FALSE)+
  geom_line() +
  facet_wrap( ~ Group) +
  scale_x_continuous("Non-centrality parameter") +
  scale_y_continuous("Difference in number of rejections vs Holm-Shaffer") +
  scale_linetype_discrete("Method") +
  theme(legend.position="bottom")
dev.off()




